@@ -1,4 +1,4 @@
-SELECT DISTINCT
+SELECT
     imaged_moment_uuid,
     image_reference_uuid,
     anno.observation_uuid AS observation_uuid,
@@ -18,6 +18,8 @@ SELECT DISTINCT
     chief_scientist,
     dive_number,
     video_sequence_name,
+    video_width,
+    video_height,
     camera_platform,
     depth_meters,
     latitude,
@@ -30,4 +32,6 @@ SELECT DISTINCT
 FROM
     annotations anno INNER JOIN associations assoc ON anno.observation_uuid = assoc.observation_uuid
 WHERE
+    assoc.link_name = 'bounding box' AND
+    assoc.link_value LIKE '{{%}}' AND
     {filters}
\ No newline at end of file
