@@ -8,9 +8,11 @@ Distributed under MIT license. See license.txt for more information.
 
 import numpy as np
 import pyqtgraph as pg
-from PyQt6 import QtCore
+from PyQt6 import QtCore, QtWidgets, QtGui
 
 from vars_gridview.lib.log import LOGGER
+from vars_gridview.lib.m3.operations import get_kb_concepts, get_kb_parts
+from vars_gridview.lib.settings import SettingsManager
 
 
 class BoundingBox(pg.RectROI):
@@ -22,6 +24,7 @@ class BoundingBox(pg.RectROI):
         rect,
         localization,
         verifier,
+        image_mosaic,
         color=(255, 0, 0),
         label="ROI",
     ):
@@ -61,6 +64,70 @@ class BoundingBox(pg.RectROI):
 
         self.localization = localization
         self.rect = rect
+        
+        self.image_mosaic = image_mosaic
+        
+        self._menu = QtWidgets.QMenu()
+        self._setup_menu()
+        
+        self.setAcceptedMouseButtons(QtCore.Qt.MouseButton.LeftButton)
+        self.sigClicked.connect(lambda bbox, ev: self.rect.clicked.emit(self.rect, ev))  # Pass click event to rect
+    
+    def _setup_menu(self):
+        """
+        Set up the context menu for the bounding box.
+        """
+        self._menu.addAction("Change concept", self._do_change_concept)
+        self._menu.addAction("Change part", self._do_change_part)
+        self._menu.addSeparator()
+        self._menu.addAction("Delete", self._do_delete)
+    
+    def contextMenuEvent(self, ev):
+        """
+        Show the context menu.
+        """
+        self._menu.popup(ev.screenPos())
+
+    def _do_delete(self):
+        """
+        Delete (clicked from context menu).
+        """
+        self.image_mosaic.select(self.rect)
+        self.image_mosaic.delete_selected()
+    
+    def _do_change_concept(self):
+        """
+        Change concept (clicked from context menu).
+        """
+        concept, ok = QtWidgets.QInputDialog.getItem(
+            self.image_mosaic._graphics_view,
+            "Change concept",
+            "Concept:",
+            get_kb_concepts()
+        )
+        
+        if not ok:
+            return
+        
+        self.image_mosaic.select(self.rect)
+        self.image_mosaic.apply_label(concept, "")
+    
+    def _do_change_part(self):
+        """
+        Change part (clicked from context menu).
+        """
+        part, ok = QtWidgets.QInputDialog.getItem(
+            self.image_mosaic._graphics_view,
+            "Change part",
+            "Part:",
+            get_kb_parts()
+        )
+        
+        if not ok:
+            return
+        
+        self.image_mosaic.select(self.rect)
+        self.image_mosaic.apply_label("", part)
 
     def check_bounds(self):
         x, y = self.pos()
@@ -126,7 +193,7 @@ class BoundingBox(pg.RectROI):
         if self.textItem is None:
 
             self.textItem = pg.TextItem(
-                text=self.label, color=np.array(self.color) / 2, fill=(200, 200, 200)
+                text=self.label, color=(255, 255, 255), fill=(70, 70, 70)
             )
             self.textItem.setPos(x, y)
             self.view.addItem(self.textItem)
@@ -134,12 +201,6 @@ class BoundingBox(pg.RectROI):
             self.textItem.setText(self.label)
             self.textItem.setPos(x, y)
 
-    def mouseClickEvent(self, ev):
-        if (
-            ev.button() == QtCore.Qt.MouseButton.RightButton
-        ):  # Capture and ignore right clicks
-            pass
-
 
 class BoxHandler:
     def __init__(
@@ -171,13 +232,15 @@ class BoxHandler:
                 box.update_label()
 
     def add_annotation(self, obj_idx, rect):
+        settings = SettingsManager.get_instance()
+        q_color = QtGui.QColor.fromString(settings.selection_highlight_color.value)
         for idx, localization in enumerate(rect.localizations):
             if localization.deleted:
                 continue
             selected_loc = idx == obj_idx
 
             # Set color of the box
-            color = (52, 161, 235)  # Nice light blue
+            color = q_color.getRgb()
             if not selected_loc:
                 avg = int(sum(color) / len(color))
                 color = (avg, avg, avg)  # Grayscale equivalent of original color
@@ -195,9 +258,10 @@ class BoxHandler:
                     self.view_box,
                     [xmin, height - ymin],
                     [xmax - xmin, -1 * (ymax - ymin)],
-                    rect,
+                    localization.rect,
                     localization,
                     self.verifier,
+                    self.image_mosaic,
                     color=color,
                     label=label,
                 )
