@@ -23,4 +23,13 @@ BASE_QUERY_FILE = ASSETS_DIR / "base_query.sql"
 LOG_DIR = ROOT_DIR / "logs"
 
 # Preferred image type
-IMAGE_TYPE = "image/png"
\ No newline at end of file
+IMAGE_TYPE = "image/png"
+
+# Appearance defaults
+LABEL_FONT_SIZE_DEFAULT = 8
+SELECTION_HIGHLIGHT_COLOR_DEFAULT = "#34a1eb"
+
+# Video player defaults
+SHARKTOPODA_HOST_DEFAULT = "::1"
+SHARKTOPODA_OUTGOING_PORT_DEFAULT = 8800
+SHARKTOPODA_INCOMING_PORT_DEFAULT = 8801
