@@ -139,6 +139,12 @@ class AnnosaurusClient(M3Client):
     @needs_auth
     def delete_observation(self, observation_uuid: str) -> requests.Response:
         return self.delete(f'/observations/{observation_uuid}')
+    
+    def get_imaged_moment(self, imaged_moment_uuid: str) -> requests.Response:
+        return self.get(f"/imagedmoments/{imaged_moment_uuid}")
+    
+    def get_image_reference(self, image_reference_uuid: str) -> requests.Response:
+        return self.get(f"/imagereferences/{image_reference_uuid}")
 
 
 class VampireSquidClient(M3Client):
@@ -184,6 +190,9 @@ class VARSKBServerClient(M3Client):
 
     def get_concepts(self) -> requests.Response:
         return self.get("/concept")
+    
+    def get_concept(self, concept: str) -> requests.Response:
+        return self.get(f"/concept/{concept}")
 
     def get_parts(self) -> requests.Response:
         return self.get("/phylogeny/taxa/organism part")
