@@ -16,6 +16,8 @@ from PyQt6 import QtCore, QtGui, QtWidgets
 from vars_gridview.lib.annotation import VARSLocalization
 from vars_gridview.lib.m3 import operations
 from vars_gridview.lib.log import LOGGER
+from vars_gridview.lib.settings import SettingsManager
+from vars_gridview.lib.util import get_timestamp
 
 
 class RectWidget(QtWidgets.QGraphicsWidget):
@@ -102,6 +104,13 @@ class RectWidget(QtWidgets.QGraphicsWidget):
         
         return self.deleted
 
+    @property
+    def imaged_moment_uuid(self) -> str:
+        """
+        Get the UUID of the imaged moment associated with this rect widget.
+        """
+        return self.localization.imaged_moment_uuid
+
     @property
     def observation_uuid(self) -> str:
         """
@@ -145,22 +154,7 @@ class RectWidget(QtWidgets.QGraphicsWidget):
         recorded_timestamp = self.video_data.get("index_recorded_timestamp", None)
 
         # Get annotation video time index
-        annotation_datetime = None
-        if recorded_timestamp is not None:
-            annotation_datetime = recorded_timestamp
-        elif elapsed_time_millis is not None:
-            annotation_datetime = video_start_datetime + datetime.timedelta(
-                milliseconds=int(elapsed_time_millis)
-            )
-        elif timecode is not None:
-            hours, minutes, seconds, frames = map(int, timecode.split(":"))
-            annotation_datetime = video_start_datetime + datetime.timedelta(
-                hours=hours, minutes=minutes, seconds=seconds
-            )
-        else:
-            raise ValueError("No video time index found")
-
-        return annotation_datetime
+        return get_timestamp(video_start_datetime, recorded_timestamp, elapsed_time_millis, timecode)
 
     def toqimage(self, img):
         height, width, bytesPerComponent = img.shape
@@ -222,19 +216,14 @@ class RectWidget(QtWidgets.QGraphicsWidget):
         return orpixmap
 
     def paint(self, painter, option, widget):
+        # Get app settings
+        settings = SettingsManager.get_instance()
+        
         pen = QtGui.QPen()
         pen.setWidth(1)
         pen.setBrush(QtCore.Qt.GlobalColor.black)
         painter.setPen(pen)
 
-        # very simple selection and annotation logic
-        if self.is_selected:
-            fill_color = QtCore.Qt.GlobalColor.green
-        elif self.is_verified:
-            fill_color = QtCore.Qt.GlobalColor.yellow
-        else:
-            fill_color = QtCore.Qt.GlobalColor.darkGray
-
         def color_for_concept(concept: str):
             hash = sum(map(ord, concept)) << 5
             color = QtGui.QColor()
@@ -250,7 +239,7 @@ class RectWidget(QtWidgets.QGraphicsWidget):
                     int(self.boundingRect().width() + 4),
                     int(self.boundingRect().height() + 4),
                 ),
-                QtGui.QColor(61, 174, 233, 255),
+                QtGui.QColor.fromString(settings.selection_highlight_color.value),
             )
 
         # Fill background if verified
@@ -299,7 +288,7 @@ class RectWidget(QtWidgets.QGraphicsWidget):
         
         # Set font
         font = QtGui.QFont(
-            'Arial', int(self.zoom * self.labelheight * 0.5), QtGui.QFont.Weight.Bold, False
+            'Arial', settings.label_font_size.value, QtGui.QFont.Weight.Bold, False
         )
         painter.setFont(font)
 
