@@ -16,10 +16,14 @@ This was inspired from a PyQt4 GraphicsGridLayout example found here:
 http://synapses.awardspace.info/pages-scripts/python/pages/python-pyqt_qgraphicsview-thumbnails-grid.py.html
 """
 
-import datetime
+import argparse
 import json
+import logging
 import os
 import sys
+from threading import Thread
+from time import sleep
+from uuid import UUID, uuid4
 import webbrowser
 from pathlib import Path
 from typing import Optional, Tuple
@@ -27,17 +31,18 @@ from typing import Optional, Tuple
 import cv2
 import pyqtgraph as pg
 import qdarkstyle
+from sharktopoda_client.client import SharktopodaClient
+from sharktopoda_client.dto import Localization
 from PyQt6 import QtCore, QtGui, QtWidgets
 
 from vars_gridview.lib import constants, m3, raziel, sql
 from vars_gridview.lib.boxes import BoxHandler
 from vars_gridview.lib.image_mosaic import ImageMosaic
-from vars_gridview.lib.log import LOGGER
+from vars_gridview.lib.log import LOGGER, AppLogger
 from vars_gridview.lib.m3.operations import (
     get_kb_concepts,
     get_kb_parts,
-    get_vars_imaged_moment,
-    get_videos_at_datetime,
+    get_imaged_moment,
 )
 from vars_gridview.lib.settings import SettingsManager
 from vars_gridview.lib.sort_methods import (
@@ -58,11 +63,13 @@ from vars_gridview.lib.util import parse_iso
 from vars_gridview.lib.widgets import RectWidget
 from vars_gridview.ui.LoginDialog import LoginDialog
 from vars_gridview.ui.QueryDialog import QueryDialog
+from vars_gridview.ui.SortDialog import SortDialog
 from vars_gridview.ui.settings.SettingsDialog import SettingsDialog
 
 # Define main window class from template
 CWD = Path(__file__).parent
 ASSETS_DIR = CWD.parent / "assets"
+ICONS_DIR = ASSETS_DIR / "icons"
 UI_FILE_PATH = ASSETS_DIR / "gridview.ui"
 WindowTemplate, TemplateBaseClass = pg.Qt.loadUiType(UI_FILE_PATH)
 
@@ -88,6 +95,8 @@ class MainWindow(TemplateBaseClass):
     """
     Main application window.
     """
+    
+    sharktopodaConnected = QtCore.pyqtSignal()
 
     def __init__(self, app):
         QtWidgets.QMainWindow.__init__(self)
@@ -119,18 +128,25 @@ class MainWindow(TemplateBaseClass):
         self.cached_moment_concepts = (
             {}
         )  # Cache for imaged moment -> set of observed concepts
+        
+        self.sharktopoda_client = None  # Sharktopoda client
+        self.sharktopoda_connected = False  # Whether the Sharktopoda client is connected
 
         # Connect signals to slots
         self.ui.discardButton.clicked.connect(self.delete)
         self.ui.clearSelections.clicked.connect(self.clear_selected)
         self.ui.labelSelectedButton.clicked.connect(self.update_labels)
         self.ui.zoomSpinBox.valueChanged.connect(self.update_zoom)
-        self.ui.sortMethod.currentTextChanged.connect(self.update_layout)
+        # self.ui.sortMethod.currentTextChanged.connect(self.update_layout)
         self.ui.hideLabeled.stateChanged.connect(self.update_layout)
         self.ui.styleComboBox.currentTextChanged.connect(self._style_gui)
         self.ui.openVideo.clicked.connect(self.open_video)
+        self.ui.sortButton.clicked.connect(self._sort_widgets)
+        
+        self._settings = SettingsManager.get_instance()
+        self._settings.label_font_size.valueChanged.connect(self.update_layout)
 
-        self.settings_dialog = SettingsDialog(self)
+        self.settings_dialog = SettingsDialog(self._setup_sharktopoda_client, self.sharktopodaConnected, parent=self)
 
         self._launch()
 
@@ -154,11 +170,14 @@ class MainWindow(TemplateBaseClass):
         # Set up the label combo boxes
         self._setup_label_boxes()
 
-        # Set up the sort method combo box
-        self._setup_sort_methods()
-
         # Set up the menu bar
         self._setup_menu_bar()
+        
+        # Set up Sharktopoda client
+        try:
+            self._setup_sharktopoda_client()
+        except Exception as e:
+            LOGGER.warning(f"Could not set up Sharktopoda client: {e}")
 
         LOGGER.info("Launch successful")
 
@@ -185,10 +204,9 @@ class MainWindow(TemplateBaseClass):
         username, password, raziel_url = login  # unpack the credentials
 
         # Update the Raziel URL setting
-        settings = SettingsManager.get_instance()
-        if settings.raz_url.value != raziel_url:
+        if self._settings.raz_url.value != raziel_url:
             LOGGER.debug(f"Updating Raziel URL setting to {raziel_url}")
-            settings.raz_url.value = raziel_url
+            self._settings.raz_url.value = raziel_url
 
         # Authenticate Raziel + get endpoint data
         endpoints = self._auth_raziel(raziel_url, username, password)
@@ -249,6 +267,8 @@ class MainWindow(TemplateBaseClass):
         file_menu = menu_bar.addMenu("&File")
 
         settings_action = QtGui.QAction("&Settings", self)
+        settings_icon = QtGui.QIcon(str(ICONS_DIR / "gear-solid.svg"))
+        settings_action.setIcon(settings_icon)
         settings_action.setShortcut("Ctrl+,")
         settings_action.triggered.connect(self._open_settings)
         file_menu.addAction(settings_action)
@@ -256,9 +276,51 @@ class MainWindow(TemplateBaseClass):
         query_menu = menu_bar.addMenu("&Query")
 
         query_action = QtGui.QAction("&Query", self)
+        query_icon = QtGui.QIcon(str(ICONS_DIR / "magnifying-glass-solid.svg"))
+        query_action.setIcon(query_icon)
         query_action.setShortcut("Ctrl+Q")
         query_action.triggered.connect(self._do_query)
         query_menu.addAction(query_action)
+    
+        # Create a menu with icons on the left-side of the main window
+        toolbar = QtWidgets.QToolBar()
+        toolbar.setObjectName("toolbar")
+        toolbar.addAction(settings_action)
+        toolbar.addAction(query_action)
+        toolbar.setIconSize(QtCore.QSize(16, 16))
+        self.addToolBar(QtCore.Qt.ToolBarArea.LeftToolBarArea, toolbar)
+        
+    
+    def _setup_sharktopoda_client(self):
+        """
+        Create the Sharktopoda video player client.
+        """
+        if self.sharktopoda_client is not None:  # stop the sharktopoda client UDP server
+            self.sharktopoda_client.stop_server()
+        
+        try:
+            self.sharktopoda_client = SharktopodaClient(
+                self._settings.sharktopoda_host.value, 
+                self._settings.sharktopoda_outgoing_port.value, 
+                self._settings.sharktopoda_incoming_port.value
+            )
+        except Exception as e:
+            LOGGER.error(f"Could not create Sharktopoda client: {e}")
+            return
+        
+        for handler in LOGGER.handlers:
+            self.sharktopoda_client.logger.addHandler(handler)
+            self.sharktopoda_client._udp_client.logger.addHandler(handler)
+            self.sharktopoda_client._udp_server.logger.addHandler(handler)
+        
+        ok = self.sharktopoda_client.connect()
+        self.sharktopoda_connected = ok
+        
+        if not ok:
+            LOGGER.warning("Could not connect to Sharktopoda")
+            return
+        
+        self.sharktopodaConnected.emit()
 
     def _open_settings(self):
         """
@@ -273,6 +335,26 @@ class MainWindow(TemplateBaseClass):
         m3.setup_from_settings()
         sql.connect_from_settings()
 
+    def _sort_widgets(self):
+        """
+        Open the sort dialog and apply a sort method to the rect widgets.
+        """
+        if not self.loaded:
+            QtWidgets.QMessageBox.warning(self, "Not Loaded", "No results are loaded, so sorting cannot be performed.")
+            return
+        
+        # Show a sort dialog
+        sort_dialog = SortDialog(parent=self)
+        ok = sort_dialog.exec()
+        if not ok:
+            return
+        method = sort_dialog.method
+        if method is None:
+            return
+        
+        self.image_mosaic.sort_rect_widgets(method)
+        self.image_mosaic.render_mosaic()
+
     def _do_query(self):
         """
         Perform a query based on the filter string.
@@ -310,9 +392,8 @@ class MainWindow(TemplateBaseClass):
         self.image_mosaic.hide_to_review = False
         self.image_mosaic._hide_labeled = self.ui.hideLabeled.isChecked()
 
-        # Render
-        sort_method = self.ui.sortMethod.currentData()
-        self.image_mosaic.sort_rect_widgets(sort_method)
+        default_sort_method = RecordedTimestampSort
+        self.image_mosaic.sort_rect_widgets(default_sort_method)
         self.image_mosaic.render_mosaic()
 
         # Show some stats about the images and annotations
@@ -351,14 +432,6 @@ class MainWindow(TemplateBaseClass):
             QtWidgets.QCompleter.CompletionMode.PopupCompletion
         )
 
-    def _setup_sort_methods(self):
-        """
-        Populate the sort method combo box
-        """
-        self.ui.sortMethod.clear()
-        for method in ENABLED_SORT_METHODS:
-            self.ui.sortMethod.addItem(method.NAME, userData=method)
-
     def _restore_gui(self):
         """
         Restore window size and splitter states
@@ -369,14 +442,22 @@ class MainWindow(TemplateBaseClass):
             self.restoreState(GUI_SETTINGS.value("windowState"))
             self.ui.splitter1.restoreState(GUI_SETTINGS.value("splitter1state"))
             self.ui.splitter2.restoreState(GUI_SETTINGS.value("splitter2state"))
+            self.ui.styleComboBox.setCurrentText(GUI_SETTINGS.value("style"))
 
     def _save_gui(self):
         GUI_SETTINGS.setValue("geometry", self.saveGeometry())
         GUI_SETTINGS.setValue("windowState", self.saveState())
         GUI_SETTINGS.setValue("splitter1state", self.ui.splitter1.saveState())
         GUI_SETTINGS.setValue("splitter2state", self.ui.splitter2.saveState())
+        GUI_SETTINGS.setValue("style", self.ui.styleComboBox.currentText())
 
     def update_labels(self):
+        if not self.loaded:
+            QtWidgets.QMessageBox.warning(
+                self, "Not Loaded", "No results are loaded, so labels cannot be applied."
+            )
+            return
+        
         concept = self.ui.labelComboBox.currentText()
         part = self.ui.partComboBox.currentText()
 
@@ -410,6 +491,9 @@ class MainWindow(TemplateBaseClass):
     @QtCore.pyqtSlot()
     def delete(self):
         if not self.loaded:
+            QtWidgets.QMessageBox.warning(
+                self, "Not Loaded", "No results are loaded, so deletions cannot be performed."
+            )
             return
 
         to_delete = self.image_mosaic.get_selected()
@@ -441,11 +525,11 @@ class MainWindow(TemplateBaseClass):
         if not self.loaded:
             return
 
-        method = self.ui.sortMethod.currentData()
+        # method = self.ui.sortMethod.currentData()
         self.image_mosaic.hide_discarded = False
         self.image_mosaic.hide_to_review = False
         self.image_mosaic._hide_labeled = self.ui.hideLabeled.isChecked()
-        self.image_mosaic.sort_rect_widgets(method)
+        # self.image_mosaic.sort_rect_widgets(method)
         self.image_mosaic.render_mosaic()
 
     @QtCore.pyqtSlot(int)
@@ -476,11 +560,15 @@ class MainWindow(TemplateBaseClass):
             self.image_mosaic.select(rect, clear=not ctrl)
 
         # Remove highlight from the last selected ROI
+        needs_autorange = True
         if self.last_selected_rect is not None:
             self.box_handler.clear()
             self.last_selected_rect.is_last_selected = False
             self.last_selected_rect.update()
 
+            # Check if new rect image is different than last rect image
+            needs_autorange = rect.image is not self.last_selected_rect.image
+
         # Update the last selection
         rect.is_last_selected = True
         rect.update()
@@ -491,6 +579,8 @@ class MainWindow(TemplateBaseClass):
         if rect_full_image is None:
             return
         self.box_handler.roi_detail.setImage(cv2.cvtColor(rect_full_image, cv2.COLOR_BGR2RGB))
+        if needs_autorange:
+            self.box_handler.view_box.autoRange()
         self.box_handler.add_annotation(rect.localization_index, rect)
 
         # Add localization data to the panel
@@ -513,12 +603,12 @@ class MainWindow(TemplateBaseClass):
             ]
         )
 
-        # Update VARS observations label
+        # Update observations label
         imaged_moment_uuid = rect.localization.imaged_moment_uuid
         if imaged_moment_uuid in self.cached_moment_concepts:  # cache hit
             concepts = self.cached_moment_concepts[imaged_moment_uuid]
         else:  # cache miss
-            vars_moment_data = get_vars_imaged_moment(
+            vars_moment_data = get_imaged_moment(
                 rect.localization.imaged_moment_uuid
             )
             concepts = sorted(
@@ -536,47 +626,74 @@ class MainWindow(TemplateBaseClass):
         if not self.last_selected_rect:
             QtWidgets.QMessageBox.warning(self, "No ROI Selected", "No ROI selected.")
             return
+        
+        rect = self.last_selected_rect
 
-        # Get the annotation recorded datetime
-        annotation_datetime = self.last_selected_rect.annotation_datetime()
+        # Get the annotation imaged moment UUID
+        imaged_moment_uuid = rect.imaged_moment_uuid
         
-        # Get the annotation video reference UUID and start timestamp
-        video_reference_uuid = self.last_selected_rect.video_data.get("video_reference_uuid", None)
-        proxy_mp4 = self.last_selected_rect.video_data.get("proxy_mp4", None)
-        if proxy_mp4 is None:
+        # Get the annotation MP4 video data
+        mp4_video_data = self.image_mosaic.moment_mp4_data.get(imaged_moment_uuid, None)
+        if mp4_video_data is None:
             QtWidgets.QMessageBox.warning(
                 self, "Missing Video", "ROI lacks MP4 video."
             )
             return
-            
-        video_start_datetime = proxy_mp4.get("start_timestamp", None)
         
-        # Exit if we don't have what we need
-        if annotation_datetime is None or video_reference_uuid is None or video_start_datetime is None:
-            QtWidgets.QMessageBox.warning(
-                self, "Missing Info", "ROI lacks necessary information to link video."
-            )
-            return
-        
-        # Get the MP4 video reference from the image mosaic, if available
-        mp4_video_reference = self.image_mosaic.video_reference_uuid_to_mp4_video_reference.get(video_reference_uuid, None)
-        if mp4_video_reference is None:
-            QtWidgets.QMessageBox.warning(
-                self, "Missing Video", "ROI lacks MP4 video."
-            )
-            return
+        mp4_video = mp4_video_data["video"]
+        mp4_video_reference = mp4_video_data["video_reference"]
 
         mp4_video_url = mp4_video_reference.get("uri", None)
+        mp4_start_timestamp = parse_iso(mp4_video["start_timestamp"])
+
+        # Get the annotation timestamp
+        annotation_datetime = self.image_mosaic.moment_timestamps[imaged_moment_uuid]
 
         # Compute the timedelta between the annotation and video start
-        annotation_timedelta = annotation_datetime - parse_iso(video_start_datetime)
+        annotation_timedelta = annotation_datetime - mp4_start_timestamp
 
-        # Open the MP4 video at the computed timedelta (in seconds)
-        annotation_seconds = max(annotation_timedelta.total_seconds(), 0)
-        url = mp4_video_url + "#t={},{}".format(
-            annotation_seconds, annotation_seconds + 1e-3
-        )  # "pause" at the annotation
-        webbrowser.open(url)
+        if not self.sharktopoda_connected:
+            # Open the MP4 video at the computed timedelta (in seconds)
+            annotation_seconds = max(annotation_timedelta.total_seconds(), 0)
+            url = mp4_video_url + "#t={},{}".format(
+                annotation_seconds, annotation_seconds + 1e-3
+            )  # "pause" at the annotation
+            webbrowser.open(url)
+            return
+        
+        # Open the video in Sharktopoda 2
+        annotation_milliseconds = max(annotation_timedelta.total_seconds() * 1000, 0)
+        video_reference_uuid = UUID(mp4_video_reference["uuid"])
+        
+        def color_for_concept(concept: str):
+            hash = sum(map(ord, concept)) << 5
+            color = QtGui.QColor()
+            color.setHsl(round((hash % 360) / 360 * 255), 255, 217, 255)
+            return color
+        
+        localization = Localization(
+            uuid=uuid4(),
+            concept=rect.localization.concept,
+            elapsed_time_millis=annotation_milliseconds,
+            x=rect.localization.x,
+            y=rect.localization.y,
+            width=rect.localization.width,
+            height=rect.localization.height,
+            duration_millis=1000,
+            color=color_for_concept(rect.localization.concept).name()
+        )
+        
+        def show_localization():
+            sleep(0.5)  # A hack, since Sharktopoda 2 crashes if you send it a command too soon
+            self.sharktopoda_client.seek_elapsed_time(video_reference_uuid, annotation_milliseconds)
+            self.sharktopoda_client.clear_localizations(video_reference_uuid)
+            self.sharktopoda_client.add_localizations(video_reference_uuid, [localization])
+        
+        self.sharktopoda_client.open(
+            video_reference_uuid, 
+            mp4_video_url, 
+            callback=show_localization
+        )
 
     @QtCore.pyqtSlot()
     def _style_gui(self):
@@ -623,9 +740,32 @@ def init_settings():
     settings.sql_database = ("sql/database", str, constants.SQL_DATABASE_DEFAULT)
 
     settings.raz_url = ("m3/raz_url", str, constants.RAZIEL_URL_DEFAULT)
+    
+    settings.label_font_size = ("appearance/label_font_size", int, constants.LABEL_FONT_SIZE_DEFAULT)
+    settings.selection_highlight_color = ("appearance/selection_highlight_color", str, constants.SELECTION_HIGHLIGHT_COLOR_DEFAULT)
+    
+    settings.sharktopoda_host = ("video/sharktopoda_host", str, constants.SHARKTOPODA_HOST_DEFAULT)
+    settings.sharktopoda_outgoing_port = ("video/sharktopoda_outgoing_port", int, constants.SHARKTOPODA_OUTGOING_PORT_DEFAULT)
+    settings.sharktopoda_incoming_port = ("video/sharktopoda_incoming_port", int, constants.SHARKTOPODA_INCOMING_PORT_DEFAULT)
+
+
+def parse_args():
+    """
+    Parse command line arguments.
+    """
+    parser = argparse.ArgumentParser(description="VARS Gridview")
+    parser.add_argument("-v", "--verbose", action="store_true", help="Verbose logging to console")
+    return parser.parse_args()
 
 
 def main():
+    # Parse command line arguments
+    args = parse_args()
+
+    # Set up logging
+    if args.verbose:
+        AppLogger.get_instance().set_stream_level(logging.DEBUG)
+
     # Create the Qt application
     app = QtWidgets.QApplication(sys.argv)
     app.setApplicationName(constants.APP_NAME)
